from rtlsdr.rtlsdrtcp import RtlSdrTcpClient
from configparser import ConfigParser
import predict 
from json import loads
from logging import getLogger, basicConfig, INFO, info, DEBUG, debug, warning
from datetime import datetime
from multiprocessing.pool import Pool
from time import sleep
import sys

from classes.RtlTcpClient import RtlTCPClient
from classes.APT import APT

basicConfig(level=INFO)
logger = getLogger(__name__)

config = ConfigParser()
config.read("config.ini")

def predict_satellite_passes():
    # Predicts satellite passes and returns value until next
    qth = (config.getfloat("LOCATION", "LAT"), config.getfloat("LOCATION", "LON"), config.getfloat("LOCATION", "ALT"))
    with open("tle.json", "r") as f:
        tle = loads(f.read())

    passes = []
    for i in tle:
        observation = predict.observe(tle[i]["tle"], qth)
        info(f"Observing {observation['name']}, ID: {observation['norad_id']}")
        transit= predict.transits(tle[i]["tle"], qth)
        transit = next(transit)
        
        delta = datetime.now().fromtimestamp(transit.start) - datetime.now()
        passes.append({ 
                        "name": observation["name"],
                        "freq": int(float(tle[i]["freq"])*10**6),
                        "start":  datetime.now().fromtimestamp(transit.start).isoformat(),
                        "duration": round(transit.duration()/60, 2) ,
                        "max_elevation": round(transit.peak()["elevation"], 2),
                        "passing_soon": delta.seconds if delta.seconds < 10*60 else False,
                        "passing_now": delta.seconds if delta.seconds <= 0 else False 
        })
             
        
    return passes

def upload_to_discord():
    # Uploads the image to discord
    pass

def generate_image():
    # Generates an image for the satellite pass
    pass


def setup():
    # Connect to a remote rtl_tcp server
    sdr = RtlSdrTcpClient(hostname=config.get("RADIO", "HOSTNAME"), port=config.getint("RADIO", "PORT"))

    # Configure the SDR
    sdr.set_sample_rate(config.getint("RADIO", "SAMPLERATE"))
    sdr.set_gain('auto') # Change if you know what you're doing
    #TODO: Add bandwidth

def satellite_tracking(satellite):
    # Brings all the functions together
    if satellite["passing_soon"] or satellite["passing_now"]:
        if not satellite["passing_now"]:
            info(f"Passing soon: {satellite['name']}")
            sleeper = satellite["passing_soon"]
            while sleeper > 0:
                if sleeper % 100 == 0:
                    warning(f"Passing soon: {satellite['name']}, {sleeper} seconds left")
                elif sleeper <= 10:
                    warning(f"{sleeper}:{satellite['name']}")
                sleeper -= 1
                sleep(1)
        
        info(f"Passing now: {satellite['name']}")
        
        if "NOAA" in satellite["name"]:
            tcp = RtlTCPClient()
            tcp.tune(satellite["freq"])
            data = tcp.receive_data(satellite["duration"]*60)
            
            apt = APT(data)
            apt.decode(f"{satellite['name']}_{datetime.now().isoformat()}.png")
        # Everything else goes here
    
    else:
        sleeper = datetime.fromisoformat(satellite["start"]).timestamp() - datetime.now().timestamp()
        
        info(f"{satellite['name']} is not passing soon, sleeping until then :) (sleeping for {str(round(sleeper/(60*60),1)) + ' hours' if sleeper>60*60 else str(round(sleeper/60)) + ' minutes'})")
        debug(satellite)
        sleep(sleeper)

# Example of parallel running function
# Remember to add to pool
#   p.apply_async(debugging)
# And to increase process count
        # def parallel_func_example():
        #     while True:
        #         print("Hey, I can run right here, forever!")
        #         sleep(1)
        
def debugging(freq:float, duration:float, name:str):
    tcp = RtlTCPClient()
    tcp.tune(freq)
    data = tcp.receive_data(duration)
    print("Done receiving")        
    apt = APT(data)
    apt.decode(f"{name}_{datetime.now().isoformat()}.png")

def main():
    # Predict passes and get ready to loop
    # debugging(137.1e6, 10, "NOAA-19")
    while True:
        passes = predict_satellite_passes()
        
        # Pool to run for all sattelites
        p = Pool(processes=len(passes)+1)
        
        p.map(satellite_tracking, passes)

        # When everything is finished, sleep for 5 seconds and restart      
        sleep(5)
         
    #     ## TODO: 
    #     # Record audio
    #     # Generate image
    #     # Upload to discord
    #     # Glue them together

if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        print("Exiting...")
        sys.exit(0)